#coding: utf-8
from __future__ import with_statement
from contextlib import closing
import httplib

# urllib2 doesn't support timeouts for python 2.5

def request(method, url, data=None, headers={}, timeout=None):
    host_port = url.split('/')[2]
    timeout_set = False
    try:
        connection = httplib.HTTPConnection(host_port, timeout = timeout)
        timeout_set = True
    except TypeError:
        connection = httplib.HTTPConnection(host_port)

    with closing(connection):
        if not timeout_set:
            connection.connect()
            connection.sock.settimeout(timeout)
            timeout_set = True

        connection.request(method, url, data, headers)
        response = connection.getresponse()
        return (response.status, response.read())



#coding: utf-8
import os, sys
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(PROJECT_ROOT, '..')))

DEBUG = True
DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = 'db.sqlite'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite'
    }
}
ROOT_URLCONF = 'urls'
TEMPLATE_DIRS = (os.path.join(PROJECT_ROOT, 'templates'),)

SECRET_KEY = '123'

# мой ключ для домена example.com
YANDEX_MAPS_API_KEY = "AOmQeUwBAAAAxp1AOQIAlTD0uSCCp1ukf-GmyqC5tbvCGAgAAAAAAAAAAAAeazHdQX6EPpcOJnuUPR-QlXyJPQ=="

INSTALLED_APPS=(
    'yandex_maps',
    'test_app',
#    'devserver',
    'south',
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        db.rename_column('yandex_maps_mapandaddress', 'longtitude', 'longitude')

    def backwards(self, orm):
        db.rename_column('yandex_maps_mapandaddress', 'longitude', 'longtitude')

    models = {
        'yandex_maps.mapandaddress': {
            'Meta': {'object_name': 'MapAndAddress'},
            'address': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'latitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'longitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['yandex_maps']


from django.shortcuts import get_object_or_404
from django.views.generic.simple import direct_to_template
from django.conf import settings
from yandex_maps.models import MapAndAddress

def yandex_map(request, map_id):
    map = get_object_or_404(MapAndAddress, id=map_id)
    api_key = settings.YANDEX_MAPS_API_KEY
    return direct_to_template(request, 'yandex_maps/map.html', {
                                 'longitude': map.longitude,
                                 'latitude': map.latitude,
                                 'api_key': api_key,
                                 'detail': 15,
                                 'address' : map.address,
                             })



#coding: utf-8
from django.core.urlresolvers import reverse
from django.test import TestCase

class YandexMapTest(TestCase):
    fixtures=['yandex_maps']

    def _check_url(self, url_name, status=200, **kwargs):
        url = reverse(url_name, kwargs=kwargs)
        response = self.client.get(url)
        self.assertEqual(response.status_code, status)
        return response

    def test_views(self):
        self._check_url('yandex_map', map_id=1)
        self._check_url('yandex_map', 404, map_id=4)



#coding: utf-8
from django import template
from django.contrib.gis.geos import Point
from django.utils.html import conditional_escape
from yandex_maps.models import MapAndAddress, get_static_map_url
register = template.Library()

def _url_for(address, *args, **kwargs):
    if isinstance(address, Point):
        return get_static_map_url(address.x, address.y, *args, **kwargs)

    if not isinstance(address, MapAndAddress):
        address, created = MapAndAddress.objects.get_or_create(address=address)
    try:
        return address.get_map_url(*args, **kwargs)
    except:
        return ''

@register.filter
def static_map_url(address, params=None):
    '''Фильтр, который возвращает URL картинки с картой.
    Можно применять к объекту класса MapAndAddress, к строке с адресом
    или к экземпляру Point из GeoDjango (например, PointField с координатами).

    Параметры: ширина, высота, уровень детализации - через запятую без пробелов.

    Пример:

        <img src='{{ address|static_map_url:"300,200" }}'>

    '''
    data = [] if params is None else params.split(",")
    return _url_for(address, *data)


@register.simple_tag
def yandex_map(address, width, height, zoom = 14, attrs=''):
    '''Тег, который выводит <img> с картой.

    Параметры:

        1. адрес (строка с адресом, объект класса MapAndAddress или Point из GeoDjango)
        2. ширина
        3. высота
        4. уровень детализации (по умолчанию = 14)

    Пример:

        {% yandex_map "Санкт-Петербург, ул. Бумажная 4" 300 200 8 %}

    '''
    url = _url_for(address, width, height, zoom)
    return "<img src='%s' width='%s' height='%s' alt='%s' %s />" % (
             url, width, height, conditional_escape(address), attrs)



#coding: utf-8
from unittest import TestCase
from yandex_maps.api import _get_coords, get_map_url

RESPONSE = u"""<?xml version="1.0" encoding="utf-8"?>
<ymaps xmlns="http://maps.yandex.ru/ymaps/1.x" xmlns:x="http://www.yandex.ru/xscript">
  <GeoObjectCollection>
    <metaDataProperty xmlns="http://www.opengis.net/gml">
      <GeocoderResponseMetaData xmlns="http://maps.yandex.ru/geocoder/1.x">
        <request>Екатеринбург, Свердлова 27</request>
        <found>1</found>
        <results>10</results>
      </GeocoderResponseMetaData>
    </metaDataProperty>
    <featureMember xmlns="http://www.opengis.net/gml">
      <GeoObject xmlns="http://maps.yandex.ru/ymaps/1.x">
        <metaDataProperty xmlns="http://www.opengis.net/gml">
          <GeocoderMetaData xmlns="http://maps.yandex.ru/geocoder/1.x">
            <kind>house</kind>
            <text>Россия, Свердловская область, Екатеринбург, улица Свердлова, 27</text>
            <precision>number</precision>
            <AddressDetails xmlns="urn:oasis:names:tc:ciq:xsdschema:xAL:2.0">
              <Country>
                <CountryName>Россия</CountryName>
                <AdministrativeArea>
                  <AdministrativeAreaName>Свердловская область</AdministrativeAreaName>
                  <Locality>
                    <LocalityName>Екатеринбург</LocalityName>
                    <Thoroughfare>
                      <ThoroughfareName>улица Свердлова</ThoroughfareName>
                      <Premise>
                        <PremiseNumber>27</PremiseNumber>
                      </Premise>
                    </Thoroughfare>
                  </Locality>
                </AdministrativeArea>
              </Country>
            </AddressDetails>
          </GeocoderMetaData>
        </metaDataProperty>
        <boundedBy xmlns="http://www.opengis.net/gml">
          <Envelope>
            <lowerCorner>60.599720 56.852332</lowerCorner>
            <upperCorner>60.607931 56.856830</upperCorner>
          </Envelope>
        </boundedBy>
        <Point xmlns="http://www.opengis.net/gml">
          <pos>60.603826 56.854581</pos>
        </Point>
      </GeoObject>
    </featureMember>
  </GeoObjectCollection>
</ymaps>
""".encode('utf8')

UNKNOWN_ADDRESS = u'''<?xml version="1.0" encoding="utf-8"?>
<ymaps xmlns="http://maps.yandex.ru/ymaps/1.x" xmlns:x="http://www.yandex.ru/xscript">
  <GeoObjectCollection>
    <metaDataProperty xmlns="http://www.opengis.net/gml">
      <GeocoderResponseMetaData xmlns="http://maps.yandex.ru/geocoder/1.x">
        <request>Екатеринбург, Свердлова 87876</request>
        <found>0</found>
        <results>10</results>
      </GeocoderResponseMetaData>
    </metaDataProperty>
  </GeoObjectCollection>
</ymaps>
'''.encode('utf8')

TEST_API_KEY = 'vasia'
COORDS = (u'60.603826', u'56.854581')
MAP_URL = 'http://static-maps.yandex.ru/1.x/?ll=60.6038260,56.8545810&size=200,300&z=5&l=map&pt=60.6038260,56.8545810&key=vasia'

class GeocodeParsingTest(TestCase):
    def test_parsing(self):
        self.assertEqual(_get_coords(RESPONSE), COORDS)

    def test_unknown(self):
        self.assertEqual(_get_coords(UNKNOWN_ADDRESS), (None, None,))


class MapUrlTest(TestCase):
    def test_map_url(self):
        url = get_map_url(TEST_API_KEY, COORDS[0], COORDS[1], 5, 200, 300)
        self.assertEqual(url, MAP_URL)



#coding: utf-8
from django.db import models
from django.conf import settings
from yandex_maps import api

YANDEX_KEY = getattr(settings, 'YANDEX_MAPS_API_KEY', None)

def get_static_map_url(longitude, latitude, width=None, height=None, detail_level=14):
    """
    Возвращает адрес статичной карты с учетом настроек в settings.py
    """
    w = int(width) if width else settings.YANDEX_MAPS_W
    h = int(height) if height else settings.YANDEX_MAPS_H
    detail_level = int(detail_level)
    return api.get_map_url(YANDEX_KEY, longitude, latitude, detail_level, w, h)


class MapAndAddress(models.Model):
    address = models.CharField(u'Адрес', max_length=255, blank=True, db_index=True)
    longitude = models.FloatField(u'Долгота', null=True, blank=True)
    latitude = models.FloatField(u'Широта', null=True, blank=True)

    def get_detail_level(self):
        return 5

    def get_map_url(self, width=None, height=None, detail_level = 5):
        if YANDEX_KEY is None:
            return ""
        return get_static_map_url(self.longitude, self.latitude, width, height, detail_level)

    def fill_geocode_data(self):
        if YANDEX_KEY is not None:
            self.longitude, self.latitude = api.geocode(settings.YANDEX_MAPS_API_KEY, self.address)

    def save(self, *args, **kwargs):
        # fill geocode data if it is unknown
        if self.pk or (self.longitude is None) or (self.latitude is None):
            self.fill_geocode_data()
        super(MapAndAddress, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.address



#!/usr/bin/env python
from distutils.core import setup

# hack for utf8 long_description support
import sys
reload(sys).setdefaultencoding("UTF-8")

version='0.6'

setup(
    name = 'yandex-maps',
    version = version,
    author = 'Mikhail Korobov',
    author_email = 'kmike84@gmail.com',
    url = 'https://bitbucket.org/kmike/yandex-maps/',

    description = 'Yandex.Maps API python wrapper with optional django integration.',
    long_description = open('README.rst').read().decode('utf8') + open('CHANGES.rst').read().decode('utf8'),
    license = 'MIT license',
    requires = ['django (>=1.2)'],

    packages=['yandex_maps', 'yandex_maps.templatetags', 'yandex_maps.migrations'],
    package_data={'yandex_maps': ['templates/yandex_maps/*']},

    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



# coding: utf-8

"""
Yandex.Maps API wrapper
"""

import xml.dom.minidom
import urllib
from yandex_maps import http

STATIC_MAPS_URL = 'http://static-maps.yandex.ru/1.x/?'
GEOCODE_URL = 'http://geocode-maps.yandex.ru/1.x/?'

def get_map_url(api_key, longitude, latitude, zoom, width, height):
    ''' returns URL of static yandex map '''
    params = [
       'll=%0.7f,%0.7f' % (float(longitude), float(latitude),),
       'size=%d,%d' % (width, height,),
       'z=%d' % zoom,
       'l=map',
       'pt=%0.7f,%0.7f' % (float(longitude), float(latitude),),
       'key=%s' % api_key
    ]
    return STATIC_MAPS_URL + '&'.join(params)

def geocode(api_key, address, timeout=2):
    ''' returns (longtitude, latitude,) tuple for given address '''
    try:
        xml = _get_geocode_xml(api_key, address, timeout)
        return _get_coords(xml)
    except IOError:
        return None, None

def _get_geocode_xml(api_key, address, timeout=2):
    url = _get_geocode_url(api_key, address)
    status_code, response = http.request('GET', url, timeout=timeout)
    return response

def _get_geocode_url(api_key, address):
    if isinstance(address, unicode):
        address = address.encode('utf8')
    params = urllib.urlencode({'geocode': address, 'key': api_key})
    return GEOCODE_URL + params

def _get_coords(response):
    try:
        dom = xml.dom.minidom.parseString(response)
        pos_elem = dom.getElementsByTagName('pos')[0]
        pos_data = pos_elem.childNodes[0].data
        return tuple(pos_data.split())
    except IndexError:
        return None, None
